package cn.com.qmth.stmms.biz.report.utils.module;

import java.util.HashMap;
import java.util.Map;

import net.sf.json.JSONObject;

import org.apache.commons.lang.StringUtils;

import cn.com.qmth.stmms.biz.exam.model.ExamStudent;
import cn.com.qmth.stmms.biz.report.model.ReportSubjectClass;
import cn.com.qmth.stmms.biz.report.service.ReportSubjectClassService;
import cn.com.qmth.stmms.biz.report.utils.Module;
import cn.com.qmth.stmms.biz.report.utils.ReportContext;
import cn.com.qmth.stmms.biz.report.utils.unit.RangeCounter;
import cn.com.qmth.stmms.biz.utils.SpringContextHolder;

public class SubjectClassRangeModule implements Module {

    private Map<String, RangeCounter> counters;

    private ReportContext context;

    public SubjectClassRangeModule(ReportContext context) {
        this.counters = new HashMap<String, RangeCounter>();
        this.context = context;
    }

    public void process(ExamStudent student) {
        if (student.isUpload() && !student.isAbsent() && !student.isBreach() && student.getSubject() != null) {
            findCounter(getKey(student)).process(student.getTotalScore());
        }
    }

    public String getKey(ExamStudent student) {
        return student.getSubjectCode() + "\t" + StringUtils.trimToEmpty(student.getClassName());
    }

    public void save() {
        ReportSubjectClassService service = SpringContextHolder.getBean(ReportSubjectClassService.class);
        for (String key : this.counters.keySet()) {
            RangeCounter counter = counters.get(key);
            String s[] = key.split("\t");
            String subjectCode = s[0];
            String className = s[1];
            ReportSubjectClass r = service.findOne(this.context.getExamId(), subjectCode, className);
            
            JSONObject rangeLevel = new JSONObject();
            rangeLevel.accumulate("highScore", counter.countGeAndLt(100 - context.getHighValueConfig(), null));
            rangeLevel.accumulate("lowScore", counter.countGeAndLt(null, context.getLowValueConfig()));
            r.setRangeLevel(rangeLevel.toString());
            
            JSONObject scoreRange = new JSONObject();
            for (int i = 0; i <= 100; i++) {
                scoreRange.accumulate(String.valueOf(i), counter.countGeAndLt((double) i, (double) (i + 1)));
            }
            r.setScoreRange(scoreRange.toString());
            service.save(r);
        }

    }

    private RangeCounter findCounter(String key) {
        RangeCounter counter = counters.get(key);
        if (counter == null) {
            counter = new RangeCounter();
            counter.interval = context.getRangeInterval();
            counters.put(key, counter);
        }
        return counter;
    }

}
